@@ -65,6 +65,7 @@ let s_ident n =
 	| "is" -> "_is"
 	| "as" -> "_as"
 	| "int" -> "_int"
+	| "const" -> "_const"
 	| "getTimer" -> "_getTimer"
 	| "typeof" -> "_typeof"
 	| "parseInt" -> "_parseInt"
@@ -280,6 +281,14 @@ let rec gen_call ctx e el =
 		gen_value ctx e1;
 		spr ctx " as ";
 		gen_value ctx e2;
+	| TLocal "__int__" , [e] ->
+		spr ctx "int(";
+		gen_value ctx e;
+		spr ctx ")";
+	| TLocal "__float__" , [e] ->
+		spr ctx "Number(";
+		gen_value ctx e;
+		spr ctx ")";
 	| TLocal "__typeof__", [e] ->
 		spr ctx "typeof ";
 		gen_value ctx e;
@@ -293,6 +302,16 @@ let rec gen_call ctx e el =
 		gen_value ctx e;
 		print ctx ") %s.push(%s)" ret tmp;
 		b();
+	| TLocal "__hkeys__", [e] ->
+		let ret = (match ctx.in_value with None -> assert false | Some r -> r) in
+		print ctx "%s = new Array()" ret;
+		newline ctx;
+		let b = save_locals ctx in
+		let tmp = define_local ctx "$k" in
+		print ctx "for(var %s : String in " tmp;
+		gen_value ctx e;
+		print ctx ") %s.push(%s.substr(1))" ret tmp;
+		b();
 	| TLocal "__new__", e :: args ->
 		spr ctx "new ";
 		gen_value ctx e;
@@ -428,7 +447,10 @@ and gen_expr ctx e =
 		b();
 	| TFunction f ->
 		let h = gen_function_header ctx None f [] e.epos in
+		let old = ctx.in_static in
+		ctx.in_static <- true;
 		gen_expr ctx (block f.tf_expr);
+		ctx.in_static <- old;
 		h();
 	| TCall (e,el) ->
 		gen_call ctx e el
@@ -629,7 +651,7 @@ and gen_value ctx e =
 		)
 	in
 	match e.eexpr with
-	| TCall ({ eexpr = TLocal "__keys__" },_) ->
+	| TCall ({ eexpr = TLocal "__keys__" },_) | TCall ({ eexpr = TLocal "__hkeys__" },_) ->
 		let v = value true in
 		gen_expr ctx e;
 		v()
@@ -660,6 +682,8 @@ and gen_value ctx e =
 		let v = value true in
 		gen_expr ctx e;
 		v()
+	| TBlock [e] ->
+		gen_value ctx e
 	| TBlock el ->
 		let v = value true in
 		let rec loop = function
@@ -719,7 +743,7 @@ let generate_field ctx static f =
 	let rights = (if static then "static " else "") ^ (if public then "public" else "protected") in
 	let p = ctx.curclass.cl_pos in
 	match f.cf_expr with
-	| Some { eexpr = TFunction fd } when f.cf_set = F9MethodAccess ->
+	| Some { eexpr = TFunction fd } when f.cf_set = MethodCantAccess ->
 		print ctx "%s " rights;
 		let rec loop c =
 			match c.cl_super with
@@ -807,7 +831,7 @@ let generate_class ctx c =
 		let f = { f with
 			cf_name = snd c.cl_path;
 			cf_public = true;
-			cf_set = F9MethodAccess;
+			cf_set = MethodCantAccess;
 		} in
 		ctx.constructor_block <- true;
 		generate_field ctx false f;
@@ -1016,10 +1040,11 @@ let ident_rights ctx id =
 	| A3MName (id,r) ->
 		let name = ident ctx id in
 		(match As3code.iget ctx.as3_namespaces r with
-		| A3NNamespace i when As3code.iget ctx.as3_idents i = "http://www.adobe.com/2006/flex/mx/internal" -> false, "$" ^ name
-		| A3NPublic _ | A3NNamespace _ -> false , name
-		| _ -> true , name)
-	| _ -> false, "???"
+		| A3NNamespace i when As3code.iget ctx.as3_idents i = "http://www.adobe.com/2006/flex/mx/internal" -> false, false, "$" ^ name
+		| A3NPublic _ | A3NNamespace _ -> false , false, name
+		| A3NProtected _ -> false, true, name
+		| _ -> true , false, name)
+	| _ -> false, false, "???"
 
 let rec create_dir acc = function
 	| [] -> ()
@@ -1060,14 +1085,17 @@ let has_getset ml f m =
 let gen_method ctx ch name mt =
 	let m = As3code.iget ctx.as3_method_types (As3parse.no_nz mt) in
 	let ret = (match m.mt3_ret with
-		| None -> "Void"
+		| None -> if name = "new" then "Void" else "Dynamic"
 		| Some t -> s_type_path (type_path ctx t)
 	) in
 	let p = ref 0 in
 	let params = List.map (fun a ->
 		let name = (match m.mt3_pnames with
 			| None -> "p" ^ string_of_int !p
-			| Some l -> ident ctx (List.nth l (!p))
+			| Some l ->
+				match List.nth l !p with
+				| None -> "p" ^ string_of_int !p
+				| Some i -> ident ctx i
 		) in
 		let opt_val = (match m.mt3_dparams with
 			| None -> None
@@ -1087,7 +1115,8 @@ let gen_method ctx ch name mt =
 let gen_fields ctx ch fields stat =
 	let fields = List.sort (fun f1 f2 -> compare (ident_rights ctx f1.f3_name) (ident_rights ctx f2.f3_name)) (Array.to_list fields) in
 	List.iter (fun f ->
-		let priv , name = ident_rights ctx f.f3_name in
+		let priv, prot, name = ident_rights ctx f.f3_name in
+		let rights = (if priv then "//" else "") ^ (if priv || prot then "private " else "") ^ (if stat then "static " else "") in
 		if name.[0] = '$' then
 			()
 		else match f.f3_kind with
@@ -1097,27 +1126,25 @@ let gen_fields ctx ch fields stat =
 			else
 			(match m.m3_kind with
 			| MK3Normal ->
-				IO.printf ch "\t";
-				if priv then IO.printf ch "private ";
-				if stat then IO.printf ch "static ";
-				gen_method ctx ch name m.m3_type
+				IO.printf ch "\t%s" rights;
+				gen_method ctx ch name m.m3_type				
 			| MK3Getter ->
 				let set = has_getset fields f m in
 				let set_str = if set then "" else "(default,null)" in
 				let m = As3code.iget ctx.as3_method_types (As3parse.no_nz m.m3_type) in
 				let t = (match m.mt3_ret with None -> "Dynamic" | Some t -> s_type_path (type_path ctx t)) in
-				IO.printf ch "\t%s%svar %s%s : %s;\n" (if priv then "private " else "") (if stat then "static " else "") name set_str t
+				IO.printf ch "\t%svar %s%s : %s;\n" rights name set_str t
 			| MK3Setter ->
 				let get = has_getset fields f m in
 				if not get then begin
 					let m = As3code.iget ctx.as3_method_types (As3parse.no_nz m.m3_type) in
 					let t = (match m.mt3_ret with None -> "Dynamic" | Some t -> s_type_path (type_path ctx t)) in
-					IO.printf ch "\t%s%svar %s(null,default) : %s;\n" (if priv then "private " else "") (if stat then "static " else "") name t
+					IO.printf ch "\t%svar %s(null,default) : %s;\n" rights name t
 				end;
 			)
 		| A3FVar v ->
 			let t = type_val ctx v.v3_type (Some v.v3_value) in
-			IO.printf ch "\t%s%svar %s : %s;\n" (if priv then "private " else "") (if stat then "static " else "") name t
+			IO.printf ch "\t%svar %s : %s;\n" rights name t
 		| A3FFunction _ ->
 			assert false
 		| A3FClass _ ->
