@@ -284,6 +284,7 @@ and gen_expr ctx e =
 	| TObjectDecl fl ->
 		(EObject (List.map (fun (f,e) -> f , gen_expr ctx e) fl),p)
 	| TArrayDecl el ->
+		if List.length el > 115 then error "This array declaration is too big, try to split it" e.epos;
 		call p (field p (ident p "Array") "new1") [array p (List.map (gen_expr ctx) el); int p (List.length el)]
 	| TCall (e,el) ->
 		gen_call ctx p e el
@@ -401,7 +402,7 @@ and gen_expr ctx e =
 		call p (builtin p "throw") [gen_expr ctx e]
 	| TMatch (e,_,cases,eo) ->
 		let etmp = (EVars ["@tmp",Some (gen_expr ctx e)],p) in
-		let etag = field p (ident p "@tmp") "tag" in
+		let eindex = field p (ident p "@tmp") "index" in
 		let gen_params params e =
 			match params with
 			| None ->
@@ -432,10 +433,10 @@ and gen_expr ctx e =
 		  (EBlock [
 			etmp;
 			(ESwitch (
-				etag,
+				eindex,
 				List.map (fun (cl,params,e2) ->
 					let cond = match cl with
-						| [s] -> str p s
+						| [s] -> int p s
 						| _ -> raise Exit
 					in					
 					cond , gen_params params e2
@@ -447,12 +448,12 @@ and gen_expr ctx e =
 			Exit ->
 				(EBlock [
 					etmp;
-					(EVars ["@tag",Some etag],p);
+					(EVars ["@index",Some eindex],p);
 					List.fold_left (fun acc (cl,params,e2) ->
 						let cond = (match cl with
 							| [] -> assert false
 							| c :: l ->
-								let eq c = (EBinop ("==",ident p "@tag",str p c),p) in
+								let eq c = (EBinop ("==",ident p "@index",int p c),p) in
 								List.fold_left (fun acc c -> (EBinop ("||",acc,eq c),p)) (eq c) l
 						) in
 						EIf (cond,gen_params params e2,Some acc),p
@@ -506,7 +507,6 @@ let gen_class ctx c =
 	let p = pos ctx c.cl_pos in
 	let clpath = gen_type_path p (fst c.cl_path,"@" ^ snd c.cl_path) in
 	let stpath = gen_type_path p c.cl_path in
-	let esuper = match c.cl_super with None -> null p | Some (c,_) -> gen_type_path p (fst c.cl_path,"@" ^ snd c.cl_path) in
 	let fnew = (match c.cl_constructor with
 	| Some f ->
 		(match follow f.cf_type with
@@ -543,23 +543,31 @@ let gen_class ctx c =
 		| None -> []
 		| Some (c,_) -> ["__super__", gen_type_path p c.cl_path]
 	) in
-	let estat = (EBinop ("=",
-		stpath,
-		(EObject (
+	let build (f,e) = (EBinop ("=",field p (ident p "@tmp") f,e),p) in
+	let tmp = (EVars ["@tmp",Some (call p (builtin p "new") [null p])],p) in
+	let estat = (EBinop ("=", stpath, ident p "@tmp"),p) in
+	let sfields = List.map build
+		(
 			("prototype",clpath) ::
 			PMap.fold (gen_method ctx p) c.cl_statics (fnew @ others)
-		),p)
-	),p) in
-	let eclass = (EBinop ("=",
-		clpath,
-		(EObject (PMap.fold (gen_method ctx p) c.cl_fields (fserialize :: fstring)),p)
-	),p) in
+		)
+	in
+	let eclass = (EBinop ("=", clpath, ident p "@tmp"),p) in
+	let mfields = List.map build
+		(PMap.fold (gen_method ctx p) c.cl_fields (fserialize :: fstring))
+	in
 	let emeta = (EBinop ("=",field p clpath "__class__",stpath),p) ::
 		match c.cl_path with
 		| [] , name -> [(EBinop ("=",field p (ident p "@classes") name,ident p name),p)]
 		| _ -> []
 	in
-	(EBlock ([eclass; estat; call p (builtin p "objsetproto") [clpath; esuper]] @ emeta),p)
+	let eextends = (match c.cl_super with
+		| None -> []
+		| Some (c,_) ->
+			let esuper = gen_type_path p (fst c.cl_path,"@" ^ snd c.cl_path) in
+			[call p (builtin p "objsetproto") [clpath; esuper]]
+	) in
+	(EBlock (tmp :: eclass :: mfields @ tmp :: estat :: sfields @ eextends @ emeta),p)
 
 let gen_enum_constr ctx path c =
 	ctx.curmethod <- c.ef_name;
@@ -571,6 +579,7 @@ let gen_enum_constr ctx path c =
 				(EBlock [
 					(EVars ["@tmp",Some (EObject [
 						"tag" , str p c.ef_name;
+						"index" , int p c.ef_index;
 						"args" , array p (List.map (ident p) params);
 					],p)],p);
 					call p (builtin p "objsetproto") [ident p "@tmp"; field p path "prototype"];
@@ -579,7 +588,7 @@ let gen_enum_constr ctx path c =
 			),p)
 		| _ ->
 			(EBlock [
-				(EVars ["@tmp",Some (EObject ["tag" , str p c.ef_name; "__serialize" , ident p "@tag_serialize"],p)],p);
+				(EVars ["@tmp",Some (EObject ["tag" , str p c.ef_name; "index", int p c.ef_index; "__serialize" , ident p "@tag_serialize"],p)],p);
 				call p (builtin p "objsetproto") [ident p "@tmp"; field p path "prototype"];				
 				ident p "@tmp";
 			],p)
@@ -682,7 +691,11 @@ let gen_name ctx acc t =
 		let p = pos ctx e.e_pos in
 		let name = fst e.e_path @ [snd e.e_path] in
 		let arr = call p (field p (ident p "Array") "new1") [array p (List.map (fun n -> gen_constant ctx e.e_pos (TString n)) name); int p (List.length name)] in
-		(EBinop ("=",field p (gen_type_path p e.e_path) "__ename__",arr),p) :: acc
+		let path = gen_type_path p e.e_path in
+		let setname = (EBinop ("=",field p path "__ename__",arr),p) in
+		let arr = call p (field p (ident p "Array") "new1") [array p (List.map (fun n -> gen_constant ctx e.e_pos (TString n)) e.e_names); int p (List.length e.e_names)] in
+		let setconstrs = (EBinop ("=", field p path "__constructs__", arr),p) in
+		setname :: setconstrs :: acc
 	| TClassDecl c ->
 		if c.cl_extern || c.cl_path = ([],"@Main") then
 			acc
