@@ -17,6 +17,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
 open Printf
+open Genswf
 
 type target =
 	| No
@@ -38,7 +39,6 @@ let get_full_path file =
 	Extc.get_full_path file
 
 let normalize_path p =
-	let p = ExtString.String.strip p in
 	let l = String.length p in
 	if l = 0 then
 		"./"
@@ -133,7 +133,7 @@ let read_type_path p cp =
 		let r = (try Sys.readdir dir with _ -> [||]) in
 		Array.iter (fun f ->
 			if (Unix.stat (dir ^ "/" ^ f)).Unix.st_kind = Unix.S_DIR then begin
-				if f.[0] > 'a' && f.[0] < 'z' then packages := f :: !packages
+				if f.[0] >= 'a' && f.[0] <= 'z' then packages := f :: !packages
 			end else if file_extension f = "hx" then begin
 				let c = Filename.chop_extension f in
 				if String.length c < 2 || String.sub c (String.length c - 2) 2 <> "__" then classes := c :: !classes;
@@ -144,6 +144,31 @@ let read_type_path p cp =
 
 let delete_file f = try Sys.remove f with _ -> ()
 
+let parse_hxml file =
+	let ch = (try open_in file with _ -> failwith ("File not found " ^ file)) in
+	let lines = Std.input_list ch in
+	close_in ch;
+	List.concat (List.map (fun l ->
+		let l = ExtString.String.strip l in
+		(*// disabled - need additional str.cmxa linkage
+			let renv = Str.regexp "%\\([A-Za-z0-9_]+\\)%" in
+			let l = Str.global_substitute renv (fun _ ->
+			let e = Str.matched_group 1 l in
+			try Sys.getenv e with Not_found -> "%" ^ e ^ "%"
+		) l in  *)
+		if l = "" || l.[0] = '#' then
+			[]
+		else if l.[0] = '-' then
+			try
+				let a, b = ExtString.String.split l " " in
+				[a; b]
+			with
+				_ -> [l]
+		else
+			[l]
+	) lines)
+
+
 let base_defines = !Plugin.defines
 
 exception Hxml_found
@@ -159,16 +184,18 @@ let rec process_params acc = function
 
 and init params =
 try
-	let version = 116 in
+	let version = 119 in
 	let version_str = Printf.sprintf "%d.%.2d" (version / 100) (version mod 100) in
-	let usage = "Haxe Compiler " ^ version_str ^ " - (c)2005-2007 Motion-Twin\n Usage : haxe.exe [options] <class names...>\n Options :" in
+	let usage = "Haxe Compiler " ^ version_str ^ " - (c)2005-2008 Motion-Twin\n Usage : haxe.exe [options] <class names...>\n Options :" in
 	let classes = ref [([],"Std")] in
 	let target = ref No in
-	let swf_in = ref None in
 	let xml_out = ref None in
 	let main_class = ref None in
-	let swf_version = ref 8 in
-	let swf_header = ref None in
+	let swf_infos = {
+		swf_version = 8;
+		swf_header = None;
+		swf_lib = None;
+	} in
 	let hres = Hashtbl.create 0 in
 	let cmds = ref [] in
 	let excludes = ref [] in
@@ -215,7 +242,7 @@ try
 		),"<file> : compile code to JavaScript file");
 		("-as3",Arg.String (fun dir ->
 			check_targets();
-			swf_version := 9;
+			swf_infos.swf_version <- 9;
 			Plugin.define "as3gen";
 			Typer.forbidden_packages := ["js"; "neko"];
 			target := As3 dir;
@@ -226,11 +253,11 @@ try
 			target := Swf file
 		),"<file> : compile code to Flash SWF file");
 		("-swf-version",Arg.Int (fun v ->
-			swf_version := v;
+			swf_infos.swf_version <- v;
 		),"<version> : change the SWF version (6,7,8,9)");
 		("-swf-header",Arg.String (fun h ->
 			try
-				swf_header := Some (match ExtString.String.nsplit h ":" with
+				swf_infos.swf_header <- Some (match ExtString.String.nsplit h ":" with
 				| [width; height; fps] ->
 					(int_of_string width,int_of_string height,float_of_string fps,0xFFFFFF)
 				| [width; height; fps; color] ->
@@ -240,7 +267,8 @@ try
 				_ -> raise (Arg.Bad "Invalid SWF header format")
 		),"<header> : define SWF header (width:height:fps:color)");
 		("-swf-lib",Arg.String (fun file ->
-			swf_in := Some file
+			if swf_infos.swf_lib <> None then raise (Arg.Bad "Only one SWF Library is allowed");
+			swf_infos.swf_lib <- Some file
 		),"<file> : add the SWF library to the compiled SWF");
 		("-neko",Arg.String (fun file ->
 			check_targets();
@@ -274,7 +302,7 @@ try
 			Plugin.define def;
 		),"<var> : define a conditional compilation flag");
 		("-resource",Arg.String (fun res ->
-			match ExtString.String.nsplit (ExtString.String.strip res) "@" with
+			match ExtString.String.nsplit res "@" with
 			| [file; name] ->
 				let file = (try Plugin.find_file file with Not_found -> file) in
 				let data = Std.input_file ~bin:true file in
@@ -333,28 +361,14 @@ try
 		),": display code tips");
 		("--no-output", Arg.Unit (fun() -> no_output := true),": compiles but does not generate any file");
 		("--times", Arg.Unit (fun() -> Plugin.times := true),": mesure compilation times");
+		("--no-inline", Arg.Unit (fun() -> Plugin.define "no_inline"),": disable inlining");
 	] in
 	let current = ref 0 in
 	let args = Array.of_list ("" :: params) in
 	let rec args_callback cl =
 		match List.rev (ExtString.String.nsplit cl ".") with
 		| x :: _ when String.lowercase x = "hxml" ->
-			let ch = (try open_in cl with _ -> failwith ("File not found " ^ cl)) in
-			let lines = Std.input_list ch in
-			let hxml_args = List.concat (List.map (fun l ->
-				let len = String.length l in
-				let l = (if len != 0 && l.[len - 1] = '\r' then String.sub l 0 (len-1) else l) in
-				if l = "" || l.[0] = '#' then
-					[]
-				else if l.[0] = '-' then
-					try
-						let a, b = ExtString.String.split l " " in
-						[a; b]
-					with
-						_ -> [l]
-				else
-					[l]
-			) lines) in
+			let hxml_args = parse_hxml cl in
 			let p1 = Array.to_list (Array.sub args 1 (!current - 1)) in
 			let p2 = Array.to_list (Array.sub args (!current + 1) (Array.length args - !current - 1)) in
 			if !Plugin.verbose then print_endline ("Processing HXML : " ^ cl);
@@ -392,7 +406,7 @@ try
 		   to accidentaly delete a source file. *)
 		if not !no_output && file_extension file = "swf" then delete_file file;
 		Plugin.define "flash";
-		Plugin.define ("flash"  ^ string_of_int !swf_version);
+		Plugin.define ("flash"  ^ string_of_int swf_infos.swf_version);
 	| Neko file ->
 		if not !no_output && file_extension file = "n" then delete_file file;
 		Plugin.define "neko";
@@ -422,7 +436,7 @@ try
 		| Swf file ->
 			do_auto_xml file;
 			if !Plugin.verbose then print_endline ("Generating swf : " ^ file);
-			Genswf.generate file (!swf_version) (!swf_header) (!swf_in) types hres
+			Genswf.generate file swf_infos types hres
 		| Neko file ->
 			do_auto_xml file;
 			if !Plugin.verbose then print_endline ("Generating neko : " ^ file);
@@ -477,7 +491,8 @@ with
 		if packs = [] && classes = [] then report ("No classes found in " ^ String.concat "." p) Ast.null_pos;
 		report_list (List.map (fun f -> f,"","") (packs @ classes));
 		exit 0;
-	| e -> report (Printexc.to_string e) Ast.null_pos
+	| e when (try Sys.getenv "OCAMLRUNPARAM" <> "b" with _ -> true) ->
+		report (Printexc.to_string e) Ast.null_pos
 
 ;;
 let all = Plugin.timer "other" in
