@@ -385,6 +385,7 @@ and parse_cf_rights allow_static l = parser
 	| [< '(Kwd Private,_) when not(List.mem APublic l || List.mem APrivate l); l = parse_cf_rights allow_static (APrivate :: l) >] -> l
 	| [< '(Kwd Override,_) when allow_static; l = parse_cf_rights false (AOverride :: l) >] -> l
 	| [< '(Kwd F9Dynamic,_) when not (List.mem AF9Dynamic l); l = parse_cf_rights false (AF9Dynamic :: l) >] -> l
+	| [< '(Kwd Inline,_); l = parse_cf_rights allow_static (AInline :: l) >] -> l
 	| [< >] -> l
 
 and parse_fun_name = parser
@@ -665,6 +666,13 @@ let parse code file =
 				process_token (skip_tokens false))
 		| Macro "if" ->
 			process_token (enter_macro())
+		| Macro "line" ->
+			let line = (match next_token() with
+				| (Const (Int s),_) -> int_of_string s
+				| (t,p) -> error (Unexpected t) p
+			) in
+			Lexer.cur_line := line - 1;
+			next_token();
 		| _ ->
 			tk
 
