@@ -16,8 +16,41 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
+open Ast
 open Type
 
+let rec iter f e =
+	match e.eexpr with
+	| TConst _
+	| TLocal _
+	| TEnumField _
+	| TBreak
+	| TContinue
+	| TTypeExpr _ -> ()
+	| TArray (e1,e2)
+	| TBinop (_,e1,e2)
+	| TFor (_,_,e1,e2)
+	| TWhile (e1,e2,_)
+		-> f e1; f e2
+	| TThrow e
+	| TField (e,_)
+	| TParenthesis e
+	| TUnop (_,_,e)
+	| TFunction { tf_expr = e }
+		-> f e
+	| TArrayDecl el 
+	| TNew (_,_,el)
+	| TBlock el
+		-> List.iter f el
+	| TObjectDecl el -> List.iter (fun (_,e) -> f e) el
+	| TCall (e,el) -> f e; List.iter f el
+	| TVars vl -> List.iter (fun (_,_,eo) -> match eo with None -> () | Some e -> f e) vl
+	| TIf (e,e1,e2) -> f e; f e1; (match e2 with None -> () | Some e -> f e)
+	| TSwitch (e,cases,def) -> f e; List.iter (fun (el,e) -> List.iter f el; f e) cases; (match def with None -> () | Some e -> f e)
+	| TMatch (e,_,cases,def) -> f e; List.iter (fun (_,_,e) -> f e) cases; (match def with None -> () | Some e -> f e)
+	| TTry (e,catches) -> f e; List.iter (fun (_,_,e) -> f e) catches
+	| TReturn eo -> (match eo with None -> () | Some e -> f e)
+
 let rec map f e =
 	match e.eexpr with
 	| TConst _
@@ -119,7 +152,7 @@ let local_find flag vname e =
 			true
 
 let block_vars e =
-	let add_var map v = map := PMap.add v () (!map) in
+	let add_var map v d = map := PMap.add v d (!map) in
 	let wrap e used =
 		match PMap.foldi (fun v _ acc -> v :: acc) used [] with
 		| [] -> e
@@ -133,72 +166,73 @@ let block_vars e =
 				List.map (fun v -> mk (TLocal v) t_dynamic e.epos) vars)
 			) t_dynamic e.epos
 	in
-	let rec in_fun vars used_locals e =
+	let rec in_fun vars depth used_locals e =
 		match e.eexpr with
 		| TLocal v ->
-			if PMap.mem v vars then add_var used_locals v
-		| TFunction _ ->
-			()
+			(try
+				if PMap.find v vars = depth then add_var used_locals v depth;				
+			with
+				Not_found -> ())
 		| _ ->
-			iter (in_fun vars used_locals) e
+			iter (in_fun vars depth used_locals) e
 
-	and in_loop vars e =
+	and in_loop vars depth e =
 		match e.eexpr with
 		| TVars l ->
 			{ e with eexpr = TVars (List.map (fun (v,t,e) ->
-				let e = (match e with None -> None | Some e -> Some (in_loop vars e)) in
-				add_var vars v;
+				let e = (match e with None -> None | Some e -> Some (in_loop vars depth e)) in
+				add_var vars v depth;
 				v, t, e
 			) l) }
-		| TFor (v,t,i,e) ->
-			let new_vars = PMap.add v () (!vars) in
-			{ e with eexpr = TFor (v,t,in_loop vars i,in_loop (ref new_vars) e) }
-		| TTry (e,cases) ->
-			let e = in_loop vars e in
+		| TFor (v,t,i,e1) ->
+			let new_vars = PMap.add v depth (!vars) in
+			{ e with eexpr = TFor (v,t,in_loop vars depth i,in_loop (ref new_vars) depth e1) }
+		| TTry (e1,cases) ->
+			let e1 = in_loop vars depth e1 in
 			let cases = List.map (fun (v,t,e) ->
-				let new_vars = PMap.add v () (!vars) in
-				v , t, in_loop (ref new_vars) e
+				let new_vars = PMap.add v depth (!vars) in
+				v , t, in_loop (ref new_vars) depth e
 			) cases in
-			{ e with eexpr = TTry (e,cases) }
-		| TMatch (e,t,cases,def) ->
-			let e = in_loop vars e in
+			{ e with eexpr = TTry (e1,cases) }
+		| TMatch (e1,t,cases,def) ->
+			let e1 = in_loop vars depth e1 in
 			let cases = List.map (fun (cl,params,e) ->
 				let e = (match params with
-					| None -> in_loop vars e
+					| None -> in_loop vars depth e
 					| Some l ->
 						let new_vars = List.fold_left (fun acc (v,t) ->
 							match v with
 							| None -> acc
-							| Some name -> PMap.add name () acc
+							| Some name -> PMap.add name depth acc
 						) (!vars) l in
-						in_loop (ref new_vars) e
+						in_loop (ref new_vars) depth e
 				) in
 				cl , params, e
 			) cases in
-			let def = (match def with None -> None | Some e -> Some (in_loop vars e)) in
-			{ e with eexpr = TMatch (e, t, cases, def) }
+			let def = (match def with None -> None | Some e -> Some (in_loop vars depth e)) in
+			{ e with eexpr = TMatch (e1, t, cases, def) }
 		| TBlock l ->
 			let new_vars = (ref !vars) in
-			map (in_loop new_vars) e
+			map (in_loop new_vars depth) e
 		| TFunction _ ->
 			let new_vars = !vars in
 			let used = ref PMap.empty in
-			iter (in_fun new_vars used) e;
+			iter (in_fun new_vars depth used) e;
 			let e = wrap e (!used) in
 			let new_vars = ref (PMap.foldi (fun v _ acc -> PMap.remove v acc) (!used) new_vars) in
-			map (in_loop new_vars) e
+			map (in_loop new_vars (depth + 1)) e
 		| _ ->
-			map (in_loop vars) e
+			map (in_loop vars depth) e
 	and out_loop e =
 		match e.eexpr with
 		| TFor _ | TWhile _ ->
-			in_loop (ref PMap.empty) e
+			in_loop (ref PMap.empty) 0 e
 		| _ ->
 			map out_loop e
 	in
 	out_loop e
 
-let emk e = mk e (mk_mono()) Ast.null_pos
+let emk e = mk e (mk_mono()) null_pos
 
 let block e =
 	match e.eexpr with
@@ -215,12 +249,12 @@ let stack_push useadd (c,m) =
 	emk (TCall (emk (TField (stack_e,"push")),[
 		if useadd then
 			emk (TBinop (
-				Ast.OpAdd,
-				emk (TConst (TString (Ast.s_type_path c.cl_path ^ "::"))),
+				OpAdd,
+				emk (TConst (TString (s_type_path c.cl_path ^ "::"))),
 				emk (TConst (TString m))
 			))
 		else
-			emk (TConst (TString (Ast.s_type_path c.cl_path ^ "::" ^ m)))
+			emk (TConst (TString (s_type_path c.cl_path ^ "::" ^ m)))
 	]))
 
 let stack_save_pos =
@@ -229,9 +263,9 @@ let stack_save_pos =
 let stack_restore_pos =
 	let ev = emk (TLocal exc_stack_var) in
 	[
-	emk (TBinop (Ast.OpAssign, ev, emk (TArrayDecl [])));
+	emk (TBinop (OpAssign, ev, emk (TArrayDecl [])));
 	emk (TWhile (
-		emk (TBinop (Ast.OpGte,
+		emk (TBinop (OpGte,
 			emk (TField (stack_e,"length")),
 			emk (TLocal stack_var_pos)
 		)),
@@ -242,7 +276,7 @@ let stack_restore_pos =
 				[]
 			))]
 		)),
-		Ast.NormalWhile
+		NormalWhile
 	));
 	emk (TCall (emk (TField (stack_e,"push")),[ emk (TArray (ev,emk (TConst (TInt 0l)))) ]))
 	]
