@@ -25,7 +25,9 @@ type field_access =
 	| NoAccess
 	| ResolveAccess
 	| MethodAccess of string
-	| F9MethodAccess
+	| MethodCantAccess
+	| NeverAccess
+	| InlineAccess
 
 type t =
 	| TMono of t option ref
@@ -111,13 +113,21 @@ and tclass_field = {
 	mutable cf_expr : texpr option;
 }
 
+and tclass_kind =
+	| KNormal
+	| KTypeParameter
+	| KExtension of tclass * tparams
+	| KConstant of tconstant
+	| KGeneric
+	| KGenericInstance of tclass * tparams
+
 and tclass = {
 	cl_path : module_path;
 	cl_pos : Ast.pos;
 	cl_doc : Ast.documentation;
 	cl_private : bool;
+	mutable cl_kind : tclass_kind;
 	mutable cl_extern : bool;
-	mutable cl_shadow : bool;
 	mutable cl_interface : bool;
 	mutable cl_types : (string * t) list;
 	mutable cl_super : (tclass * tparams) option;
@@ -198,8 +208,8 @@ let mk_class path pos doc priv =
 		cl_pos = pos;
 		cl_doc = doc;
 		cl_private = priv;
+		cl_kind = KNormal;
 		cl_extern = false;
-		cl_shadow = false;
 		cl_interface = false;
 		cl_types = [];
 		cl_super = None;
@@ -274,6 +284,7 @@ let rec is_parent csup c =
 		| Some (c,_) -> is_parent csup c
 
 let rec link e a b =
+	(* tell if a is is b *)
 	let rec loop t =
 		if t == a then
 			true
@@ -296,22 +307,49 @@ let rec link e a b =
 			with
 				Exit -> true
 	in
-	let rec mono_loop t =
+	(* tell if a ~= b *)
+	let rec loop2 t =
 		if t == a then
 			true
 		else match t with
-		| TMono t -> (match !t with None -> false | Some t -> loop t)
+		| TMono t -> (match !t with None -> false | Some t -> loop2 t)
 		| _ -> false
 	in
-	if mono_loop b then
-		true
-	else if loop b then
-		false
+	if loop b then
+		loop2 b
 	else
 		match b with
 		| TDynamic _ -> true
 		| _ -> e := Some b; true
 
+let map loop t =
+	match t with
+	| TMono r ->
+		(match !r with
+		| None -> t
+		| Some t -> loop t) (* erase*)
+	| TEnum (_,[]) | TInst (_,[]) | TType (_,[]) ->
+		t
+	| TEnum (e,tl) ->
+		TEnum (e, List.map loop tl)
+	| TInst (c,tl) ->
+		TInst (c, List.map loop tl)
+	| TType (t2,tl) ->
+		TType (t2,List.map loop tl)
+	| TFun (tl,r) ->
+		TFun (List.map (fun (s,o,t) -> s, o, loop t) tl,loop r)
+	| TAnon a ->
+		TAnon {
+			a_fields = PMap.map (fun f -> { f with cf_type = loop f.cf_type }) a.a_fields;
+			a_status = a.a_status;
+		}
+	| TLazy f ->
+		let ft = !f() in
+		let ft2 = loop ft in
+		if ft == ft2 then t else ft2
+	| TDynamic t2 ->
+		if t == t2 then	t else TDynamic (loop t2)
+
 (* substitute parameters with other types *)
 let apply_params cparams params t =
 	match cparams with
@@ -435,8 +473,8 @@ let has_extra_field t n = Has_extra_field (t,n)
 let error l = raise (Unify_error l)
 
 let unify_access a1 a2 =
-	a1 = a2 || (a1 = NormalAccess && (a2 = NoAccess || a2 = F9MethodAccess))
-	|| (a1 = F9MethodAccess && a2 = NoAccess)
+	a1 = a2 || (a1 = NormalAccess && (a2 = NoAccess || a2 = MethodCantAccess))
+	|| (a1 = MethodCantAccess && a2 = NoAccess)
 
 let eq_stack = ref []
 
