@@ -153,7 +153,7 @@ let as3_method_type_length m =
 	idx_opt_length m.mt3_debug_name +
 	1 +
 	(match m.mt3_dparams with None -> 0 | Some l -> 1 + sum (as3_value_length true) l) +
-	(match m.mt3_pnames with None -> 0 | Some l -> sum idx_length l)
+	(match m.mt3_pnames with None -> 0 | Some l -> sum idx_opt_length l)
 
 let list_length f l =
 	match Array.length l with
@@ -206,7 +206,7 @@ let as3_try_catch_length t =
 	idx_opt_length t.tc3_name
 
 let as3_function_length f =
-	let clen = sum As3code.length f.fun3_code in
+	let clen = Array.fold_left (fun acc op -> acc + As3code.length op) 0 f.fun3_code in
 	idx_length_nz f.fun3_id +
 	int_length f.fun3_stack_size +
 	int_length f.fun3_nregs +
@@ -391,7 +391,7 @@ let read_method_type ctx ch =
 		None
 	) in
 	let pnames = (if flags land 0x80 <> 0 then
-		Some (Array.to_list (Array.init nargs (fun _ -> index ctx.as3_idents (read_int ch))))
+		Some (Array.to_list (Array.init nargs (fun _ -> index_opt ctx.as3_idents (read_int ch))))
 	else
 		None
 	) in
@@ -528,7 +528,7 @@ let read_function ctx ch =
 	let init_scope = read_int ch in
 	let max_scope = read_int ch in
 	let size = read_int ch in
-	let code = if parse_bytecode then As3code.parse ch size else Array.to_list (Array.init size (fun _ -> A3Unk (IO.read ch))) in
+	let code = if parse_bytecode then As3code.parse ch size else Array.init size (fun _ -> A3Unk (IO.read ch)) in
 	let trys = read_list2 ch (read_try_catch ctx) in
 	let local_funs = read_list2 ch (read_field ctx) in
 	{
@@ -744,7 +744,7 @@ let write_method_type ch m =
 	| None -> ()
 	| Some l ->
 		if List.length l <> nargs then assert false;
-		List.iter (write_index ch) l
+		List.iter (write_index_opt ch) l
 
 let write_list ch f l =
 	match Array.length l with
@@ -824,9 +824,9 @@ let write_function ch f =
 	write_int ch f.fun3_nregs;
 	write_int ch f.fun3_init_scope;
 	write_int ch f.fun3_max_scope;
-	let clen = sum As3code.length f.fun3_code in
+	let clen = Array.fold_left (fun acc op -> acc + As3code.length op) 0 f.fun3_code in
 	write_int ch clen;
-	List.iter (As3code.write ch) f.fun3_code;
+	Array.iter (As3code.write ch) f.fun3_code;
 	write_list2 ch write_try_catch f.fun3_trys;
 	write_list2 ch write_field f.fun3_locals
 
@@ -910,7 +910,10 @@ let method_str ?(infos=false) ctx m =
 	(String.concat ", " (List.map (fun a ->
 		let id = (match m.mt3_pnames with
 			| None -> "p" ^ string_of_int !pcount
-			| Some l -> ident_str ctx (List.nth l !pcount)
+			| Some l -> 
+				match List.nth l !pcount with
+				| None -> "p" ^ string_of_int !pcount
+				| Some i -> ident_str ctx i
 		) in
 		let p = (match a with None -> id | Some t -> name_str ctx (id ^ " : ") t) in
 
@@ -1002,8 +1005,10 @@ let dump_function ctx ch idx f =
 	IO.printf ch "    stack:%d nregs:%d scope:%d-%d\n" f.fun3_stack_size f.fun3_nregs f.fun3_init_scope f.fun3_max_scope;
 	Array.iter (dump_field ctx ch false) f.fun3_locals;
 	Array.iter (dump_try_catch ctx ch) f.fun3_trys;
-	List.iter (fun op ->
-		IO.printf ch "    %s\n" (As3code.dump ctx op);
+	let pos = ref 0 in
+	Array.iter (fun op ->
+		IO.printf ch "%4d    %s\n" !pos (As3code.dump ctx op);
+		pos := !pos + As3code.length op;
 	) f.fun3_code;
 	IO.printf ch "\n"
 
